function [DCvv,DChh]=KaDOP(theta,phi,u,varargin)

% KaDOP computes sea surface Doppler spectrum centroid for VV and HH
% polarizations, DCvv and DChh, based on the empirical
% MTF [10.1109/TGRS.2017.2787459]
% The DC is expressed in m/s, DC[m/s]= DC[Hz]*lambda/2, where lambda is
% the radar wavelength. By default, the DC is estimated for
% Pierson-Moskowitz (PM) [10.1029/JZ069i024p05181]
% wind sea spectrum without swell.
% If significant wave height (SWH) and peak frequency (omega) are known,
% they can be specified explicitly. Swell can be added independently by
% specifying its SWH and omega. No swell by default.
%
% Conventions: DC is positive for approaching targets, DC is negative for
% receding targets, radar-to-wind(wave) azimuth is zero for upwind(wave).
%
% Syntax:
%
% theta     -- incidence angle, 0...65 [deg]
% phi       -- radar-to-wind azimuth, 0...180 [deg]
% u         -- wind speed, 3...15 m/s [m/s]
%
% Optional parameters:
%
% drift     -- Wind drift coefficient, 1.5%U by default
% swh       -- Wind sea Significant Wave Height [m], PM by default
% omega     -- Wind sea peak radial frequency [rad/s], PM by default
%
% swh_sw   -- Swell Significant Wave Height [m],
%             0 by default (note SWH=sqrt(8)*a,
%             where a is the wave amplitude)
% omega_sw -- Swell Peak radial frequency [rad/m], 0 by default
% phi_sw   -- Radar-to-swell azimuth [deg], 0 by default
%--------------------------------------------------------------------------
% Examples:
%
% subplot(221) % Plot incidence angle dependence
% theta = 0:65;
% phi = [0,90,180];
% u = 10;
% [VV,HH]=KaDOP(theta,phi,u);
% plot(theta,squeeze(VV),'-')
% ylim([-1,1])
% title('Color indicates azimuth')
% xlabel('Incidence Angle, deg')
% ylabel('Doppler Centroid, m/s')
% grid on
%
% subplot(222) % Plot radar-to-wind azimuth dependence
% theta = [30,45,60];
% phi = -180:180;
% u = 10;
% [VV,HH]=KaDOP(theta,phi,u);
% plot(phi,squeeze(VV),'-')
% ylim([-1,1])
% title('Color indicates azimuth')
% xlabel('Incidence Angle, deg')
% ylabel('Doppler Centroid, m/s')
% grid on
%
% subplot(223)  % Plot wind speed dependence
% theta = [30,45,60];
% phi = 0;
% u = 3:16;
% [VV,HH]=KaDOP(theta,phi,u);
% plot(u,squeeze(VV),'-')
% ylim([-1,1])
% title('Color indicates azimuth')
% xlabel('Incidence Angle, deg')
% ylabel('Doppler Centroid, m/s')
% grid on
%
% subplot(224) % Add some swell and plot incidence angle dependence
% theta = 0:65;
% phi = [0,90,180];
% u = 10;
% [VV,HH]=KaDOP(theta,phi,u,'swh_sw',14,'omega_sw',0.31,'phi_sw',180);
% plot(theta,squeeze(VV),'-')
% ylim([-1,1])
% title('Color indicates azimuth')
% xlabel('Incidence Angle, deg')
% ylabel('Doppler Centroid, m/s')
% grid on
%==========================================================================

g=9.8;         % Gravitational constant [m/s^2]
gamma=7.3e-5;  % Surface tension [N/m]
lambda=0.008;  % Radar wavelength [m], Ka-band by default
drift=0.015;   % Wind drift coefficient, 1.5%U by default
beta_ws=0.20;  % 3rd moment of Wind Sea spectrum (PM) parameter
beta_sw=1/16;  % 3rd moment of SWell spectrum parameter

swh=0.22*u.^2/g;
omega=0.83*g./u;

swh_sw=0;
omega_sw=0;
phi_sw=0;

zerocrosswind=0; % if this parameter set to 1,
% the MTFwindsea is replaced by MTFswell (crosswind is 0)

%------------- parse inputs --------------------
varargin=reshape(varargin',2,[])';
varargin(:,1)=lower(varargin(:,1));

%-------------- re-evaluate default value if any -------------------------
tryvar={'lambda','drift','beta_ws','beta_sw','omega','swh','omega_sw',...
    'swh_sw','phi_sw','zerocrosswind'};
for j=1:length(tryvar)
    try
        eval([tryvar{j} '=varargin{strcmpi(''' ...
            tryvar{j} ''',varargin(:,1)),2};']);
    end
end
%-------------------------------------------------------------------------
% Look geometry coefficient:
G=@(theta,phi) cosd(phi).*sind(theta) - 1i*cosd(theta); 
% Shortwave spectrum spreading function:
spread=@(phi) sech(1.0*acosd(cosd(phi))*pi/180).^2;     
% Wheighting of approaching and receiding Bragg waves:
Braggsum=@(phi) (spread(phi)-spread(phi+180))./(spread(phi)+spread(phi+180)); 
% Bragg wave phase speed:
vbr=@(theta,lambda) sqrt(g.*sind(theta)./(4*pi/lambda) + gamma*(4*pi/lambda*sind(theta).^3)); 

Mws=1; % MTF type for wind sea to be evaluated below
Msw=2; % MTF type for swell to be evaluated below
if zerocrosswind % zero crosswind MTFwindsea if necessary
    Mws=2;
end

%--------------- prepare inputs ------------------------------------
[THETA,PHI,U]=ndgrid(theta,phi,u);
[~,~,OMEGA]=ndgrid(theta,phi,omega);
[~,~,SWH]=ndgrid(theta,phi,swh);

[~,~,OMEGAsw]=ndgrid(theta,phi,omega_sw);
[~,~,SWHsw]=ndgrid(theta,phi,swh_sw);
[~,~,PHIsw]=ndgrid(theta,phi,phi_sw);
%-------------------------------------------------------------------

%------------- non-polarized Doppler = Bragg + sea surface current --------
nonpolDoppler = vbr(THETA,lambda) .* Braggsum(PHI) + drift*U.*cosd(PHI).*sind(THETA);

%------------- Total = non-pol + polarized Long-Wave-Induced part (wind sea + swell) --------
DCvv=nonpolDoppler + ...
    beta_ws*real(G(THETA,PHI) .* MTF(THETA,PHI,U,coeffs(Mws,1)))/g .* SWH.^2 .* OMEGA.^3 + ...
    beta_sw*real(G(THETA,PHIsw) .* MTF(THETA,PHIsw,U,coeffs(Msw,1)))/g .* SWHsw.^2 .* OMEGAsw.^3;

DChh=nonpolDoppler + ...
    beta_ws*real(G(THETA,PHI) .* MTF(THETA,PHI,U,coeffs(Mws,2)))/g .* SWH.^2 .* OMEGA.^3 + ...
    beta_sw*real(G(THETA,PHIsw) .* MTF(THETA,PHIsw,U,coeffs(Msw,2)))/g .* SWHsw.^2 .* OMEGAsw.^3;


function M=MTF(theta,phi,u,C)
% evaluate complex MTF using C-coefficients
% 1st column of C-matrix is Bijk, 2nd column is Cijk from Equation (A1),
% Table A1 or A2

Mabs=evalfun(theta,phi,u,C(:,1));
Mphs=evalfun(theta,phi,u,C(:,2));
M=exp(Mabs).*Mphs./abs(Mphs);

function pp=evalfun(theta,phi,u,C)
% evaluate magnitude or phasor of MTF
% C is column, either Bijk or Cijk

pp = C(1) + ...
    C(2) .* theta.^1 + ...
    C(3) .* theta.^2 + ...
    C(4) .* theta.^3 + ...
    C(5) .* cosd(1*phi) + ...
    C(6) .* theta.^1 .* cosd(1*phi) + ...
    C(7) .* theta.^2 .* cosd(1*phi) + ...
    C(8) .* theta.^3 .* cosd(1*phi) + ...
    C(9) .* cosd(2*phi) + ...
    C(10) .* theta.^1 .* cosd(2*phi) + ...
    C(11) .* theta.^2 .* cosd(2*phi) + ...
    C(12) .* theta.^3 .* cosd(2*phi) + ...
    C(13) .* log(u).^1 + ...
    C(14) .* theta.^1 .* log(u).^1 + ...
    C(15) .* theta.^2 .* log(u).^1 + ...
    C(16) .* theta.^3 .* log(u).^1 + ...
    C(17) .* cosd(1*phi) .* log(u).^1 + ...
    C(18) .* theta.^1 .* cosd(1*phi) .* log(u).^1 + ...
    C(19) .* theta.^2 .* cosd(1*phi) .* log(u).^1 + ...
    C(20) .* theta.^3 .* cosd(1*phi) .* log(u).^1 + ...
    C(21) .* cosd(2*phi) .* log(u).^1 + ...
    C(22) .* theta.^1 .* cosd(2*phi) .* log(u).^1 + ...
    C(23) .* theta.^2 .* cosd(2*phi) .* log(u).^1 + ...
    C(24) .* theta.^3 .* cosd(2*phi) .* log(u).^1;

function C = coeffs(type,pp)

% set of coefficients, Table A1 and A2
% 1st column is Bijk, 2nd column is Cijk from Equation (A1).
% type=1 for wind sea (Table A1),
% type=2 for swell (Table A2),
% pp=1 for VV-polarization,
% pp=2 for HH-polarization.

WindSeaVV =[+2.037368e+00, -9.991774e-01-1.859445e-03i
    -9.956181e-03, +9.995403e-02-3.728707e-02i
    +1.733240e-03, -9.495314e-04+5.073520e-04i
    -2.110994e-05, -1.742060e-06+2.930913e-06i
    -1.704388e-02, -2.062522e-03+4.317005e-03i
    -4.002570e-02, -2.021244e-02+1.328154e-01i
    +2.213287e-03, +1.037791e-03-5.526796e-03i
    -1.778161e-05, -1.183648e-05+4.932378e-05i
    -2.933537e-02, -5.651327e-05+1.289564e-03i
    +2.755026e-02, +7.638659e-02+7.101499e-02i
    +1.382417e-03, -3.141920e-03-2.127452e-03i
    -2.811759e-05, +3.360741e-05+1.363174e-05i
    -2.637003e-01, -1.300697e-03+6.335937e-04i
    +2.457828e-02, -1.060972e-02+4.969400e-03i
    -1.537867e-03, -2.108491e-05-1.405381e-05i
    +1.667354e-05, +2.373730e-06-1.623276e-06i
    +1.342060e-02, +4.740406e-04-8.386239e-04i
    +1.791006e-02, +9.982368e-03-1.343944e-02i
    -1.048575e-03, -4.634691e-04+1.129914e-03i
    +9.158551e-06, +5.153546e-06-1.134140e-05i
    +1.809446e-02, +2.879613e-04-3.980226e-04i
    +8.255341e-03, -2.309667e-02-1.347916e-02i
    -1.286835e-03, +9.359817e-04+5.873901e-04i
    +1.827908e-05, -1.056345e-05-5.154716e-06i];

WindSeaHH =[+2.038368e+00, -9.999579e-01-2.003675e-03i
    +6.742867e-02, +1.401092e-01-3.822135e-02i
    -1.544673e-03, -2.832742e-03+6.391936e-04i
    +1.167191e-05, +1.755927e-05-1.325959e-06i
    -1.716876e-02, -2.510170e-03+5.669125e-03i
    -2.064313e-02, -1.886127e-03+1.301061e-01i
    +1.172491e-03, +2.217910e-04-5.440821e-03i
    -6.111610e-06, -2.769183e-06+5.317919e-05i
    -2.939264e-02, +1.738649e-03+1.255492e-03i
    +4.007160e-03, +3.758102e-02+7.395083e-02i
    +1.482772e-03, -1.072406e-03-2.254102e-03i
    -2.163604e-05, +8.151756e-06+1.559167e-05i
    -2.643806e-01, -8.840229e-04+6.209692e-04i
    -1.240919e-02, -3.155538e-02+3.907412e-03i
    +2.162084e-04, +8.937600e-04-1.544636e-05i
    -3.482596e-07, -6.512207e-06-4.914423e-07i
    +1.347741e-02, +7.416105e-04-1.536552e-03i
    +7.223413e-03, -2.172061e-03-1.458223e-02i
    -5.037439e-04, +1.053785e-04+1.203955e-03i
    +2.889241e-06, -9.978940e-07-1.415368e-05i
    +1.812623e-02, -6.400749e-04-4.329797e-04i
    +2.313635e-02, -5.070167e-03-1.231709e-02i
    -1.569241e-03, -5.514080e-06+5.292689e-04i
    +1.795667e-05, +8.560235e-07-4.894367e-06i];

SwellVV =[+2.037368e+00, -1.047849e+00-1.086382e-03i
    -9.956181e-03, +9.779865e-02+9.409557e-03i
    +1.733240e-03, -9.521228e-04-1.330189e-03i
    -2.110994e-05, -8.936468e-07+1.921637e-05i
    -1.704388e-02, -2.054076e-02+2.380576e-02i
    -4.002570e-02, +4.046633e-02+1.544692e-01i
    +2.213287e-03, -1.395978e-03-5.769671e-03i
    -1.778161e-05, +1.340544e-05+4.688263e-05i
    -2.933537e-02, -4.552795e-03-3.923333e-03i
    +2.755026e-02, +2.273467e-02+1.289799e-02i
    +1.382417e-03, -8.407162e-04+1.345284e-05i
    -2.811759e-05, +9.080283e-06-3.645146e-06i
    -2.637003e-01, +4.449188e-03+1.717938e-03i
    +2.457828e-02, -1.171622e-02-2.045575e-03i
    -1.537867e-03, +9.499907e-05+4.015526e-04i
    +1.667354e-05, +8.816342e-07-5.631314e-06i
    +1.342060e-02, +5.159466e-03-6.475855e-03i
    +1.791006e-02, -9.459894e-03-1.412467e-02i
    -1.048575e-03, +3.075467e-04+9.873627e-04i
    +9.158551e-06, -3.260269e-06-8.840548e-06i
    +1.809446e-02, +1.029965e-03+1.201244e-03i
    +8.255341e-03, -3.648071e-03-5.884530e-03i
    -1.286835e-03, +1.828698e-06+7.071967e-05i
    +1.827908e-05, +1.276843e-07+8.061632e-08i];

SwellHH =[+2.038368e+00, -1.070596e+00+4.617718e-04i
    +6.742867e-02, +1.422845e-01+4.036745e-03i
    -1.544673e-03, -2.882753e-03-1.021860e-03i
    +1.167191e-05, +1.838410e-05+1.433121e-05i
    -1.716876e-02, -1.404714e-02+2.765220e-02i
    -2.064313e-02, +2.884548e-02+1.580035e-01i
    +1.172491e-03, -6.833107e-04-6.044204e-03i
    -6.111610e-06, +4.112504e-06+5.471463e-05i
    -2.939264e-02, +1.196099e-02-5.905559e-03i
    +4.007160e-03, -6.952809e-03+1.881372e-02i
    +1.482772e-03, +3.991268e-04-2.664768e-04i
    -2.163604e-05, -4.235270e-06-1.228258e-06i
    -2.643806e-01, +1.676822e-02+5.227076e-05i
    -1.240919e-02, -3.573475e-02-7.998733e-04i
    +2.162084e-04, +1.083750e-03+3.168845e-04i
    -3.482596e-07, -8.535620e-06-4.213366e-06i
    +1.347741e-02, +3.305453e-03-8.652656e-03i
    +7.223413e-03, -6.991652e-03-1.631189e-02i
    -5.037439e-04, +1.321311e-04+1.143512e-03i
    +2.889241e-06, -5.730351e-07-1.266315e-05i
    +1.812623e-02, -7.689661e-03+1.684635e-03i
    +2.313635e-02, +1.171194e-02-6.082012e-03i
    -1.569241e-03, -6.270342e-04+9.248031e-05i
    +1.795667e-05, +6.716300e-06-1.181313e-08i];

M{1,1}=WindSeaVV;
M{1,2}=WindSeaHH;
M{2,1}=SwellVV;
M{2,2}=SwellHH;
C=M{type,pp};